/*
 * Copyright (c) 2008 Lu, Chao-Ming (Tetralet).  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#include "notebook.h"

// For command line options.
extern gchar* command_line;
extern gchar** parameter;
// The defalut Page Name
extern gchar *page_name;
extern gchar **page_names;
extern gboolean page_number;
extern gint current_name;
// For using transparent background
extern gboolean transparent_background;
extern gdouble background_saturation;

// For showing the Menu if right click on a vtebox.
extern GtkWidget *locale_menu;
// For setting the size when adding a new page, avoid to set it to 80x24 (libvte default size)
extern gint screen_width;
extern gint screen_height;
extern gint original_screen_width;
extern gint original_screen_height;
// For sloving the function key
extern struct KeyValue pagekeys[KEYS];
// For increase/decrease the size of font
extern gchar* current_font_name;
extern gchar *default_font_name;
gboolean update_hinets = FALSE;
gboolean keep_vtebox_size = TRUE;
// For Transparent Background
extern GtkWidget *trans_bg_item;
extern GtkWidget *adjustment;

// For Set the radio_list to System Defalut
extern GtkWidget *default_encoding;

extern GtkWidget *window;
extern GtkWidget *notebook;

GtkWidget *current_vtebox=NULL;

void add_page(gint run_once)
{
	// the component of a single page
	struct Page *new_page = g_new0(struct Page, 1);
	gint current_page_no;
	const gchar *directory = g_getenv("PWD");
	gchar* label_name = NULL;

	// create label and set the label name
	if (page_names[current_name]!=NULL)
		label_name = g_strdup(page_names[current_name++]);
	else
		label_name = g_strdup(page_name);
	if (page_number)
	{
		gchar* new_label_name = NULL;
		gint total_page = gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook));
		new_label_name = g_strdup_printf("(%d) %s", total_page+1, label_name);
		new_page->label = gtk_label_new(new_label_name);
		g_free(new_label_name);
	}
	else
		new_page->label = gtk_label_new(label_name);
	new_page->label->name = label_name;

	// create a hbox
	new_page->hbox = gtk_hbox_new(FALSE, 0);

	// create vtebox
	new_page->vtebox = vte_terminal_new();

	// Get current vtebox size, font and directory. for init a new tab.
	current_page_no = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
	if ( current_page_no != -1 && current_vtebox != NULL)
	{
		screen_width = vte_terminal_get_column_count(VTE_TERMINAL(current_vtebox));
		screen_height = vte_terminal_get_row_count(VTE_TERMINAL(current_vtebox));
		// g_debug("Trying to fix vtebox to %d x %d when showing the tab bar\n",
		//	   screen_width, screen_height);

		// Got the current page's directory
		struct Page *prev_page = (struct Page *)g_object_get_data(G_OBJECT(current_vtebox), "Data");
		directory = g_file_read_link(g_strdup_printf("/proc/%d/cwd", prev_page->pid), NULL);
		// Got the default font name
		g_free(current_font_name);
		current_font_name = g_strdup(prev_page->font_name);
		// g_debug("use the directory %s (/proc/%d/cwd)\n", directory, prev_page->pid);
	}

	// Init new page. run_once: some settings only need run once.
	// run_once only = TRUE when init LilyTerm in main().
	init_new_page(window, new_page->vtebox, run_once);
	gtk_box_pack_start(GTK_BOX(new_page->hbox), new_page->vtebox, TRUE, TRUE, 0);
	// the close page event
	g_signal_connect(G_OBJECT(new_page->vtebox), "child_exited",
			 G_CALLBACK(close_page), NULL);
	// when get focus, update `current_vtebox'
	g_signal_connect(G_OBJECT(new_page->vtebox), "grab-focus",
			 G_CALLBACK(page_grab_focuse), NULL);
// #ifdef HINTS
	// when vtebox is focuse, the size of vtebox is unchangeable.
	g_signal_connect_after(G_OBJECT(new_page->vtebox), "focus-in-event", G_CALLBACK(page_get_focuse), NULL);
	// when vtebox is focuse, the size of vtebox is changeable.
	g_signal_connect(G_OBJECT(new_page->vtebox), "focus-out-event", G_CALLBACK(page_lost_focuse), NULL);
// #endif
	// if function key pressed
	g_signal_connect(G_OBJECT(new_page->vtebox), "key-press-event",
				  G_CALLBACK(vtebox_key_press), NULL);

	// show the menu
	g_signal_connect(G_OBJECT(new_page->vtebox), "button-press-event",
			 G_CALLBACK(vtebox_button_press), NULL);

	// scrollbar
	new_page->scrollbar = gtk_vscrollbar_new(vte_terminal_get_adjustment(VTE_TERMINAL(new_page->vtebox)));
	gtk_box_pack_start(GTK_BOX(new_page->hbox), new_page->scrollbar, FALSE, FALSE, 0);

	// add the new page to notebook
	current_page_no = gtk_notebook_append_page(GTK_NOTEBOOK(notebook), new_page->hbox, new_page->label);
#ifdef ENABLE_TAB_REORDER
	gtk_notebook_set_tab_reorderable(GTK_NOTEBOOK(notebook), new_page->hbox,TRUE);
	gtk_notebook_set_tab_detachable(GTK_NOTEBOOK(notebook), new_page->hbox,FALSE);
#endif
	g_object_set(gtk_widget_get_parent(new_page->label), "can-focus", FALSE, NULL);

	// show the page bar if necessary
	if (current_page_no == 1)
		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), TRUE);

	// Execute programs in the vtebox
	//if (parameter != NULL)
	//	g_debug ("Run Command: %s %s\n", command_line, *parameter);
	new_page->pid = vte_terminal_fork_command(VTE_TERMINAL(new_page->vtebox),
						  command_line, parameter, NULL,
						  directory, TRUE, TRUE, TRUE);
	// We MUST clear command_line and parameter after run -e option.
	command_line = NULL;
	parameter = NULL;

	// for debug only
	// gchar *pid = g_strdup_printf("%d", new_page->pid);
	// gtk_label_set_text(GTK_LABEL(new_page->label), pid);
	// g_free(pid);

	// finish!
	gtk_widget_show_all(new_page->hbox);
	gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), current_page_no);
	gtk_window_set_focus(GTK_WINDOW(window), new_page->vtebox);

	// Store the new page data
	new_page->encoding = default_encoding;
	new_page->font_name = g_strdup(current_font_name);
	new_page->transparent_background = transparent_background;
	new_page->background_saturation = background_saturation;
	g_object_set_data(G_OBJECT(new_page->vtebox), "Data", new_page);
	g_object_set_data(G_OBJECT(new_page->label), "VteBox", new_page->vtebox);

	// gtk_widget_show_all(notebook);
}

void close_page (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	gint total_page=gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook));
	gint current_page=gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
	
	// g_debug("Total Page (Notebook):%d\n", total_page);

	// free the memory used by this page
	struct Page *current_data = (struct Page *)g_object_get_data(G_OBJECT(current_vtebox), "Data");
	// g_debug("Deleting Page ID: %d\n", current_data->pid);
	g_free(current_data);

	// if it is the last page, exit application!
	if (total_page == 1)
		gtk_main_quit();
	else
	{
		// remove current page
		// g_debug ("This page is going to be removed!\n");
		gtk_notebook_remove_page(GTK_NOTEBOOK(notebook), current_page);

		if ( current_page < total_page-1 )
			gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), current_page);

		// hide the page bar if necessary
		if (total_page == 2)
		{
			// hide the page bar
			gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), FALSE);
			// Yes, just set it to 1 x 1,
			// and the window will be resized to correct geometry automatically.
			gtk_window_resize(GTK_WINDOW(window), 1, 1);
		}
	}
	if (current_page < total_page-1)
		reorder_page_number(NULL, NULL);
}

void page_grab_focuse(GtkWidget *vtebox, gpointer user_data)
{
	// g_debug ("Update current_vtebox!\n");
	current_vtebox = vtebox;
	// if (current_vtebox!=NULL)
	// {
	//  struct Page *current_data = (struct Page *)g_object_get_data(G_OBJECT(current_vtebox), "Data");
	//  if (current_data!=NULL)
	//	  g_debug("Change to the %d vtebox(%d), and the pid is %d!\n",
	//		   gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)),
	//		   current_vtebox, current_data->pid);
	// }

	// if update_hinets = TRUE, we should update hints for every vtebox.
	if (update_hinets)
		window_resizable(window, current_vtebox, 1, -1);
}

// #ifdef HINTS
void page_get_focuse(GtkWidget *vtebox, gpointer user_data)
{
	// g_debug("vtebox get focuse, vtebox can NOT resize!\n");
	if (current_vtebox!=NULL && keep_vtebox_size)
		window_resizable(window, current_vtebox, 0, -1);
	else
		keep_vtebox_size = TRUE;
}

void page_lost_focuse(GtkWidget *vtebox, gpointer user_data)
{
	// g_debug("vtebox lost focuse, vtebox can be resized!\n");
	if (current_vtebox!=NULL)
		window_resizable(window, current_vtebox, 0, 1);
}
// #endif

gboolean vtebox_button_press(GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	if (event->button == 3)
	{
		struct Page *current_data = (struct Page *)g_object_get_data(G_OBJECT(current_vtebox), "Data");
		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(current_data->encoding), TRUE);
		GTK_CHECK_MENU_ITEM(trans_bg_item)->active = current_data->transparent_background;
		gtk_menu_popup(GTK_MENU(locale_menu), NULL, NULL, NULL, NULL, event->button, event->time);
		return TRUE;
	}
	return FALSE;
}

gboolean vtebox_key_press(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
	// don't check if no ctrl/shift/alt key pressed!
	if (event->state & ALL_ACCELS_MASK)
	{
		// don't check if only shift key pressed!
		if (event->state & SHIFT_ONLY_MASK)
		{
			// g_debug("ALL_ACCELS_MASK = %X\n", ALL_ACCELS_MASK);
			// g_debug("Got the function key: %s (status: %X, check: %X)\n",
			//	   gdk_keyval_name(event->keyval), event->state,
			//	   event->state|GDK_LOCK_MASK|GDK_MOD2_MASK);
			gint i, keyval, mods;
			if ((event->keyval>=GDK_a) && (event->keyval<=GDK_z))
				keyval = event->keyval - GDK_a + GDK_A;
			else
				keyval = event->keyval;
			mods = event->state|GDK_LOCK_MASK|GDK_MOD2_MASK;
			for (i=0;i<KEYS;i++)
			{
				// g_debug("Checking... key: %s (status: %X)\n",
				//	   gdk_keyval_name(pagekeys[i].key), event->state|GDK_LOCK_MASK);
				if ((mods==pagekeys[i].mods) && (keyval==pagekeys[i].key))
				{
					// deal the function key
					deal_key_press(i);
					return TRUE;
				}
			}
		}
	}
	return FALSE;
}

// Default function key:
//  [0] <Ctrl><T> = New Page
//  [1] <Ctrl><W> = Close Page
//  [2] <Ctrl><E> = Edit Page Label
//  [3] <Ctrl><V> = Past Page Label
//  [4] <Ctrl><PageUp> = Pre Page
//  [5] <Ctrl><PageDown> = Next Page
//  [6] <Ctrl><Home> = First Page
//  [7] <Ctrl><End> = Last Page
//  [8] <Ctrl><E> = Move Page Forward
//  [9] <Ctrl><C> = Move Page Backward
// [10] <Ctrl><E> = Move Page to First
// [11] <Ctrl><C> = Move Page to Last
//		<Shift><Insert> to pase clipboard.
void deal_key_press(gint type)
{
	gint current_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
	gint total_page = gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook));
	struct Page *current_data = (struct Page *)g_object_get_data(G_OBJECT(current_vtebox), "Data");

	switch (type)
	{
		case 0:
		{
			// add a new page
			add_page(0);
			break;
		}
		case 1:
		{
			// close page
			close_page (NULL,NULL,NULL);
			break;
		}
		case 2:
		{
			// edit page's label
			dialog(NULL, 1);
			break;
		}
		case 3:
		{
			// past page's label
			g_debug("Trying to rename the label whith the selected text.\n");
			break;
		}
		case 4:
		{
			// switch to pre page
			if (current_page)
				gtk_notebook_prev_page(GTK_NOTEBOOK(notebook));
			else
				gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), total_page -1);
			break;
		}
		case 5:
		{
			// switch to next page
			if (current_page == (gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook)) - 1))
				gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
			else
				gtk_notebook_next_page(GTK_NOTEBOOK(notebook));
			break;
		}
		case 6:
		{
			// switch to first page
			gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);
			break;
		}
		case 7:
		{
			// switch to last page
			gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), total_page-1);
			break;
		}
		case 8:
		{
			// move current page forward
			gtk_notebook_reorder_child(GTK_NOTEBOOK(notebook), current_data->hbox, current_page -1);
			break;
		}
		case 9:
		{
			// move current page backward
			if (current_page == (gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook)) - 1))
				 gtk_notebook_reorder_child(GTK_NOTEBOOK(notebook), current_data->hbox, 0);
			else
				gtk_notebook_reorder_child(GTK_NOTEBOOK(notebook),
							   current_data->hbox, current_page + 1);
			break;
		}
		case 10:
		{
			// move current page to first
			gtk_notebook_reorder_child(GTK_NOTEBOOK(notebook), current_data->hbox, 0);
			break;
		}
		case 11:
		{
			// move current page to last
			gtk_notebook_reorder_child(GTK_NOTEBOOK(notebook), current_data->hbox, -1);
			break;
		}
		case 12:
		{
			set_vte_font(NULL, 0);
			break;
		}
		case 13:
		{
			set_vte_font(NULL, 1);
			break;
		}
		case 14:
		{
			set_vte_font(NULL, 2);
			break;
		}
		case 15:
		case 16:
		case 17:
		case 18:
		case 19:
		case 20:
		case 21:
		case 22:
		case 23:
		case 24:
		case 25:
		case 26:
		{
			// switch to #%d page
			if (total_page > type-15)
				gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), type-15);
			break;
		}
		case 27:
		{
			vte_terminal_select_all(VTE_TERMINAL(current_vtebox));
			break;
		}
		case 28:
		{
			vte_terminal_copy_clipboard(VTE_TERMINAL(current_vtebox));
			break;
		}
		case 29:
		{
			vte_terminal_paste_clipboard(VTE_TERMINAL(current_vtebox));
			break;
		}
	}
}

void set_vte_font(GtkWidget *widget, gint type)
{
	// type = 0 : increase font size
	// type = 1 : decrease font size
	// type = 2 : reset font size
	// type = 3 : increase window size
	// type = 4 : decrease window size
	// type = 5 : reset window size
	// type = 6 : change font name & size

	struct Page *current_data = (struct Page *)g_object_get_data(G_OBJECT(current_vtebox), "Data");
	gint total_page = gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook));
	PangoFontDescription *font_desc;

	// get font name
	if (type==2 || type==5)
	{
		// restore to the default size of font
		g_free(current_font_name);
		current_font_name = g_strdup(default_font_name);
		
		// the font name of type>3 will deal later
		if (type ==2)
		{
			g_free(current_data->font_name);
			current_data->font_name = g_strdup(current_font_name);
		}
	}
	else if (type != 6)
	{
		gint fontsize;

		// Increase/Decrease the font size
		font_desc = pango_font_description_from_string(current_data->font_name);
		if (type==0 || type==3)
			fontsize = (pango_font_description_get_size(font_desc)/PANGO_SCALE)*1.15;
		else
			fontsize = (pango_font_description_get_size(font_desc)/PANGO_SCALE)/1.1;
		// g_debug("Trying to increase the font size to %d.\n", fontsize);
		pango_font_description_set_size(font_desc, fontsize*PANGO_SCALE);
		
		// the font name of type>=3 will deal later
		if (type <3)
		{
			g_free(current_data->font_name);
			current_data->font_name = pango_font_description_to_string(font_desc);
			// g_debug("The final font name = %s\n", current_data->font_name);
		}
	}

	// apply font in vtebox
	if (type==6 && total_page==1)
	{
		vte_terminal_set_font_from_string(VTE_TERMINAL(current_vtebox), current_data->font_name);
		window_resizable(window, current_vtebox, 1, -1);
		gtk_window_resize(GTK_WINDOW(window), 1, 1);
		keep_vtebox_size = FALSE;
	}
	else if (type<3 || type==6)
	{
		// trying to keep window size
		window_resizable(window, current_vtebox, 2, 1);
		vte_terminal_set_font_from_string(VTE_TERMINAL(current_vtebox), current_data->font_name);
		// if update_hinets = TRUE, we should update hints for every vtebox.
		update_hinets = TRUE;
	}
	else
	{
		gint i;
		GtkWidget *vtebox;
		struct Page *current_data;
		gchar* font_name;
		gint colum, row;

		if (type==5)
		{
			colum =	original_screen_width;
			row = original_screen_height;
		}
		else
		{
			colum = vte_terminal_get_column_count(VTE_TERMINAL(current_vtebox));
			row = vte_terminal_get_row_count(VTE_TERMINAL(current_vtebox));
		}

		if (type==5)
			font_name = g_strdup(default_font_name);
		else
			font_name = pango_font_description_to_string(font_desc);

		for (i=0;i<total_page;i++)
		{
			vtebox=(GtkWidget *)g_object_get_data(G_OBJECT(
								gtk_notebook_get_tab_label(GTK_NOTEBOOK(notebook),
									gtk_notebook_get_nth_page(
										GTK_NOTEBOOK(notebook), i))),
								"VteBox");
			current_data = (struct Page *)g_object_get_data(G_OBJECT(vtebox), "Data");
			vte_terminal_set_font_from_string(VTE_TERMINAL(vtebox), font_name); 
			vte_terminal_set_size(VTE_TERMINAL(vtebox), colum, row);
			g_free(current_data->font_name);
			current_data->font_name = g_strdup(font_name);
		}
		window_resizable(window, current_vtebox, 1, -1);
		// Yes, just set it to 1 x 1,
		// and the window will be resized to correct geometry automatically.
		gtk_window_resize(GTK_WINDOW(window), 1, 1);
	}

	if (total_page==1 || type>=3)
	{
		// update current_font_name
		g_free(current_font_name);
		current_font_name = g_strdup (current_data->font_name);
	}
}

void reorder_page_number(GtkWidget *widget, gpointer user_data)
{
	gint total_page = gtk_notebook_get_n_pages(GTK_NOTEBOOK(notebook));
	GtkWidget *vtebox;
	gchar* label_name;
	struct Page *current_data;
	gint i;

	for (i=0;i<total_page;i++)
	{
		vtebox=(GtkWidget *)g_object_get_data(G_OBJECT(
							gtk_notebook_get_tab_label(GTK_NOTEBOOK(notebook),
								gtk_notebook_get_nth_page(
									GTK_NOTEBOOK(notebook), i))),
							"VteBox");
		current_data = (struct Page *)g_object_get_data(G_OBJECT(vtebox), "Data");
		label_name = g_strdup_printf("(%d) %s", i+1, current_data->label->name);
		gtk_label_set_text(GTK_LABEL(current_data->label), label_name);
		g_free(label_name);
	}

}
